!> \file sliding-Heino_mod-testhw.f90
!! Module qui definit la loi de glissement dans les experiences Heino.
!<

!> \namespace  sliding_dragging_heino
!! Definition de la loi de glissement dans les experiences Heino.
!! \author ...
!! \date ...
!! @note Used module
!! @note   - use module3D_phy
!<

module sliding_dragging_heino

! Dfini la loi de glissement dans les expriences Heino

USE module3D_phy


implicit none


integer,dimension(nx,ny) :: mksedim      !< masque des regions sediment-hard rock
integer,dimension(nx,ny) :: mkxsedim     !< dcalle sur les demi mailles
integer,dimension(nx,ny) :: mkysedim     !< dcalle sur les demi mailles
real :: Cr                            !< coefficient glissement pour hard rock
real :: Cs                            !< coefficient glissement pour sediment
real :: coefmax
real :: coefslid
real :: coefdrag
logical,dimension(nx,ny) :: gzmx_heino     !< pour transporter la valeur dans flottab
logical,dimension(nx,ny) :: gzmy_heino
real :: longslope

real, dimension(nx,ny) :: hw_mx
real, dimension(nx,ny) :: hw_my

character(len=80) :: filin


contains
!-------------------------------------------------------------------------------

!>SUBROUTINE: init_sliding
!! Cette routine fait l'initialisation du glissement et est appele juste apres
!! la lecture du masque
!<
subroutine init_sliding


if (itracebug.eq.1)  call tracebug(' Heino: entree dans routine init_sliding')

! Cette routine fait l'initialisation du glissement et est appele juste apres
! la lecture du masque

! Cette routine fait l'initialisation du glissement

loigliss=4       ! 4 -> que glissement Heino (pas de mode dragging)
if (loigliss.eq.4) then
   flgzmx(:,:)=.false.
   flgzmy(:,:)=.false.
coefmax=hwatermax
endif
                      ! 5 -> glissement Heino + dragging

!    ecriture dans le fichier parametres
     write(42,*)'glissement Heino -  initialisation masque'
     write(42,*)'loi glissement=',loigliss
     write(42,*)'coef dependant de la hauteur d eau    coefmax=',coefmax
     write(42,*)' moyenne de la hauteur d eau a partir de 6 voisins'
     write(42,*)'----------------------------------------------------------'


! fichier mask fourni par l'intercomparaison Heino 
! on refait la lecture 

       filin=TRIM(DIRNAMEINP)//'mask.dat'
       write(42,*) 'type de socle '
       write(42,*) 'mksedim donne par ',filin
       
open(11,file=filin)
do i=1,6
      read(11,*)  ! 6 lignes de commentaires
end do


! lecture proprement dite de la carte sediment
do j=ny, 1, -1
    read(11,'(81i1)') (mksedim(i,j), i=1,nx)
end do

close(11)


! test pour une calotte axi-symetrique avec glissement Cr partout
!write(42,*) 'masque sediment mis a 1 '
!mksedim(:,:)=min(mksedim(:,:),1)

! a la frontiere le demi noeud est sediment.
! calcul du masque sur les demi mailles : frontiere -> sedim
! mkxsedim(:,:)=max(mksedim(:,:),eoshift(mksedim(:,:),shift=-1,boundary=0,dim=1))
! mkysedim(:,:)=max(mksedim(:,:),eoshift(mksedim(:,:),shift=-1,boundary=0,dim=2))

! write(42,*) 'sur les bords zone sediment, masque exterieur'


mkxsedim(:,:)=min(mksedim(:,:),eoshift(mksedim(:,:),shift=-1,boundary=0,dim=1))
mkysedim(:,:)=min(mksedim(:,:),eoshift(mksedim(:,:),shift=-1,boundary=0,dim=2))

write(42,*) 'sur les bords zone sediment, masque interieur'

mslid_mx(:,:)=mkxsedim(:,:)
mslid_my(:,:)=mkysedim(:,:)

Cr=1.e5    ! en a-1  coefficient hard rock (loi en puissance 3)
Cs=500.    ! en a-1  coefficient sediment (lineaire)


   write(42,*)'coefficients:   Cr=',Cr,'    Cs=',Cs,' (a-1)'
!  write(42,*)'glissement seulement si les 2 voisins sont au point de fusion'



      return
      end subroutine init_sliding

!________________________________________________________________________________

!> SUBROUTINE: init_dragging
!! Cette routine pour l'initialisation
!<

subroutine init_dragging

if ((loigliss.eq.5).and.(Cs.gt.1.)) then
   coefdrag=rog/Cs
   Cs=0.
   write(42,*)'passage en stream si zone sediment + fusion'
   write(42,*)'coefficient de frottement dans la zone sediment=',coefdrag
endif

end subroutine init_dragging

!________________________________________________________________________________

!> SUBROUTINE: sliding
!! Cette routine calcule le terme ddbx et ddby dans le cas Heino
!<
subroutine sliding


! cette routine calcule le terme ddbx et ddby dans le cas Heino
!      -------------------------------------------------
!      GLISSEMENT
!      DDBX et DDBY termes dus au glissement 
!      relation avec la vitesse de glissement UXB et UYB 
!      UXB=-DDBX*SDX et UYB=-DDBY*SDY
!      -------------------------------------------------

implicit none

integer :: i_moins1,j_moins1,i_plus1,j_plus1

!!$real, dimension(nx,ny) :: hw_mx
!!$real, dimension(nx,ny) :: hw_my

if (itracebug.eq.1)  call tracebug(' Heino: entree dans routine sliding')


call moy_mxmy(nx,ny,Hwater,hw_mx,hw_my)

!!$do j=1,ny
!!$   do i=1,nx
!!$      i_moins1=max(1,i-1)
!!$      j_moins1=max(1,j-1)
!!$      i_plus1=min(nx,i+1)
!!$      j_plus1=min(ny,j+1)
!!$
!!$      hw_mx(i,j)=0.25*(Hwater(i,j)+hwater(i_moins1,j)) &
!!$           + 0.125*((hwater(i_moins1,j_plus1)+hwater(i,j_plus1))   &
!!$           +       (hwater(i_moins1,j_moins1)+hwater(i,j_moins1)))
!!$
!!$      hw_my(i,j)=0.25*(Hwater(i,j)+hwater(i,j_moins1)) &
!!$           + 0.125*((hwater(i_plus1,j_moins1)+hwater(i_plus1,j))   &
!!$           +       (hwater(i_moins1,j_moins1)+hwater(i_moins1,j)))
!!$
!!$
!!$   end do
!!$end do

slidy: do j=2,NY
   do I=2,NX-1

pose_y:  if (.not.flgzmy(i,j)) then

   if (HMY(I,J).lt.1.) then        ! glace tres fine
      ddby(i,j)=0.
      mslid_my(i,j)=0

!   else if ((T(i,j,nz).lt.Tpmp(i,j,nz)).and.(T(i,j-1,nz).lt.Tpmp(i,j-1,nz))) then 
!      ddby(i,j)=0.                 ! pas de glissement seulement
!                                   ! si les 2 voisins sont froids

!   else if ((T(i,j,nz).lt.Tpmp(i,j,nz)).or.(T(i,j-1,nz).lt.Tpmp(i,j-1,nz))) then 

!      else if ((ibase(i,j).eq.1).or.(ibase(i,j-1).eq.1)) then
!   else if ((ibase(i,j).eq.1).and.(ibase(i,j-1).eq.1)) then

      else if (hw_my(i,j).lt.0.001) then
      ddby(i,j)=0.                 ! pas de glissement si au moins un seul 
                                   ! des 2 voisins est froid
      mslid_my(i,j)=0

   else                            ! glissement
!      coefslid=(hwater(i,j)+hwater(i,j-1))*0.5
      coefslid=hw_my(i,j)
      coefslid=max(coefslid,0.)
      coefslid=min(coefslid/coefmax,1.)
!      coefslid=1.

      if (mkysedim(i,j).eq.1) then  ! loi hard rock
         ddby(i,j)=Cr*slope2my(i,j)*Hmy(i,j)*coefslid

         mslid_my(i,j)=1

      else if (mkysedim(i,j).eq.2) then  ! loi sediment
         ddby(i,j)=Cs*Hmy(i,j)*coefslid

         mslid_my(i,j)=2

!         if (abs(sdy(i,j)).gt.1.e-8) then
!            j_1=max(j-1,1)
!            j1=min(j+1,ny)
!            longslope=(sdy(i,j_1)+sdy(i,j)+sdy(i,j1))/3.
!            ddby(i,j)=ddby(i,j)/sdy(i,j)*longslope
!         endif

      else
         ddby(i,j)=0.
         mslid_my(i,j)=0

      endif

   endif


endif pose_y
end do
end do slidy


slidx: do j=2,NY-1
   do I=2,NX
pose_x:  if (.not.flgzmx(i,j)) then

   if (HMx(I,J).lt.1.) then        ! glace tres fine
      ddbx(i,j)=0.
         mslid_mx(i,j)=0

!   else if ((T(i,j,nz).lt.Tpmp(i,j,nz)).and.(T(i-1,j,nz).lt.Tpmp(i-1,j,nz))) then 
!      ddbx(i,j)=0.                 ! pas de glissement seulement
!                                   ! si les 2 voisins sont froids

!   else if ((T(i,j,nz).lt.Tpmp(i,j,nz)).or.(T(i-1,j,nz).lt.Tpmp(i-1,j,nz))) then 
!     else if ((ibase(i,j).eq.1).or.(ibase(i-1,j).eq.1)) then

!      else if ((ibase(i,j).eq.1).and.(ibase(i-1,j).eq.1)) then
      else if (hw_mx(i,j).lt.0.001) then

      ddbx(i,j)=0.                 ! pas de glissement si au moins un seul 
                                   ! des 2 voisins est froid
         mslid_mx(i,j)=0



   else                            ! glissement

!      coefslid=(hwater(i,j)+hwater(i-1,j))*0.5
      coefslid=hw_mx(i,j)
      coefslid=max(coefslid,0.)
      coefslid=min(coefslid/coefmax,1.)
!      coefslid=1.

      if (mkxsedim(i,j).eq.1) then  ! loi hard rock
         ddbx(i,j)=Cr*slope2mx(i,j)*Hmx(i,j)*coefslid
         mslid_mx(i,j)=1

      else if (mkxsedim(i,j).eq.2) then  ! loi sediment
         ddbx(i,j)=Cs*Hmx(i,j)*coefslid
         mslid_mx(i,j)=2

!         if (abs(sdx(i,j)).gt.1.e-8) then
!            i_1=max(i-1,1)
!            i1=min(i+1,nx)
!            longslope=(sdx(i_1,j)+sdx(i,j)+sdx(i1,j))/3.
!            ddbx(i,j)=ddbx(i,j)/sdx(i,j)*longslope
!         endif


      else
         ddbx(i,j)=0.
         mslid_mx(i,j)=0

      endif

   endif

endif pose_x
end do
end do slidx

return
end subroutine  sliding

!-------------------------------------------------------------------------

!> SUBROUTINE: dragging
!! Defini le basal drag
!<

subroutine dragging   

! dans la zone sediment : appele si loigliss=5,
! defini le basal drag


flgzmx(:,:)=.false.
flgzmy(:,:)=.false.

betamx(:,:)=1.e5
betamy(:,:)=1.e5

if (loigliss.eq.5) then          ! dragging dans les regions mkysedim:2
   froty: do j=2,ny
      do i=2,nx-1

         if (mkysedim(i,j).eq.2) then  ! loi sediment

            ! seulement si au dessus du point de fusion 
            if ((T(i,j,nz).lt.Tpmp(i,j,nz)).or.(T(i,j-1,nz).lt.Tpmp(i,j-1,nz))) then 
            ! base froide
            else
               betamy(:,:)=coefdrag
               flgzmy(i,j)=.true.
            endif
         endif

      end do
   end do froty


   frotx: do j=2,ny-1
      do i=2,nx

         if (mkxsedim(i,j).eq.2) then  ! loi sediment

            ! seulement si au dessus du point de fusion 
            if ((T(i,j,nz).lt.Tpmp(i,j,nz)).or.(T(i-1,j,nz).lt.Tpmp(i-1,j,nz))) then 
            ! base froide
            else
               betamx(:,:)=coefdrag
               flgzmx(i,j)=.true.
            endif

         endif
      end do
   end do frotx

   gzmx(:,:)=flgzmx(:,:)
   gzmy(:,:)=flgzmy(:,:)
   gzmx_heino(:,:)=gzmx(:,:)
   gzmy_heino(:,:)=gzmy(:,:)
end if

return
end subroutine dragging


END MODULE  sliding_dragging_heino

